package usherbrooke.ift630;

import java.util.ArrayList;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;

public class Stop extends Thread {
	private String									name;
	private int										id;
	private int										maxCapacity;
	private int										indent = 0;
	private ArrayList<Passenger>					passengers;
	private ExecutorService							threads;
	private BlockingQueue<BusInformationMessage>	blockingQueue;


	// print bus info times.
	private void displayInfos(int indent) throws InterruptedException, ExitException{
		BusInformationMessage info = null;

		// lock queue, get head
		synchronized (blockingQueue) {
			info = blockingQueue.poll();
		}
		// get infos
		try {
			try {
				int time = info.getTimeToStop(this) / 1000;
				// if everything is alright, print
				Logger.getInstance().print(id,
						"\t".repeat(indent) + "[STOP] " + info.getBus(this).getNameBus() + " arrives in " + time + "s");
				// else, catch
			} catch (NullPointerException e) {}

		} catch (UnauthorizedException e) {
			// if unauth, requeue message.
			synchronized (blockingQueue) {
				blockingQueue.add(info);
			}
		}

		// loop, and ensure that every stop is reactive
		synchronized (this) {
			threads.submit(this);
			sleep(10);
			throw new ExitException();
		}
	}

	@Override
	public void run() {
		try {
			for (;;) {
				displayInfos(indent);
			}
		} catch (InterruptedException | ExitException e) {}
	}

	Stop(int id, int maxCapacity) {
		this.id = id;
		this.name = "Stop " + id;
		this.passengers = new ArrayList<Passenger>();
		this.maxCapacity = maxCapacity;
	}

	public void addPassenger(Passenger p) {
		passengers.add(p);
	}

	// return all passenger that stops at a stop in the list
	public ArrayList<Passenger> getPassengerByDest(ArrayList<Stop> list) {
		ArrayList<Passenger> res = new ArrayList<Passenger>();
		// for all given stop
		for (Stop s : list) {
			// for all passenger at this stop
			for (Passenger p : passengers) {
				// if they stop at this stop, add to res
				if (p.getDest() == s) {
					res.add(p);
				}
			}
		}

		return res;
	}

	public String getStopName() {
		return name;
	}

	public int getCurrentCapacity() {
		return passengers.size();
	}

	public int getMaxCapacity() {
		return maxCapacity;
	}

	public void printDetails(int color, int indent) {
		Logger.getInstance().print(color, "\t".repeat(indent) + "---".repeat(3) + " Stop details " + "---".repeat(3));
		Logger.getInstance().print(color, "\t".repeat(indent) + name);

		for (Passenger p : passengers) {
			p.printDetails(color, indent + 1);
		}
	}

	public void printDetails(int indent) {
		Logger.getInstance().print(id, "\t".repeat(indent) + "---".repeat(3) + " Stop details " + "---".repeat(3));
		Logger.getInstance().print(id, "\t".repeat(indent) + name);

		for (Passenger p : passengers) {
			p.printDetails(indent + 1);
		}
	}

	public void printDetails() {
		printDetails(0);
	}

	public void removePassenger(Passenger p) {
		try {
			passengers.remove(p);
		} catch (Exception e) {
			System.out.println("exception" + e.getMessage());
		}
	}

	public void setIndent(int indent) {
		this.indent = indent;
	}

	public void setBlockingQueue(BlockingQueue<BusInformationMessage> q) {
		blockingQueue = q;
	}

	public void setThreadPool(ExecutorService threads) {
		this.threads = threads;
	}
}
